@@ -46,7 +46,8 @@ extern bool secret_buffer_charset_test_codepoint(const secret_buffer_charset *cs
 #define SECRET_BUFFER_ENCODING_UTF16LE      3
 #define SECRET_BUFFER_ENCODING_UTF16BE      4
 #define SECRET_BUFFER_ENCODING_HEX          5
-#define SECRET_BUFFER_ENCODING_MAX          5
+#define SECRET_BUFFER_ENCODING_BASE64       6
+#define SECRET_BUFFER_ENCODING_MAX          6
 
 #define SECRET_BUFFER_ENCODING_IS_UNICODE(x)  \
    (  (x) == SECRET_BUFFER_ENCODING_UTF8      \
@@ -59,6 +60,7 @@ typedef struct {
    U8 *pos, *lim;
    const char *error;
    int encoding;
+   U8 pos_bit, lim_bit;
 } secret_buffer_parse;
 
 /* Initialize a parse struct, and also verify that the described span is within the
