@@ -1,7 +1,7 @@
 package Crypt::SecretBuffer::Span;
 # VERSION
 # ABSTRACT: Reference a span of bytes within a SecretBuffer
-$Crypt::SecretBuffer::Span::VERSION = '0.011';
+$Crypt::SecretBuffer::Span::VERSION = '0.012';
 use strict;
 use warnings;
 use Crypt::SecretBuffer; # loads XS methods into this package
@@ -211,6 +211,13 @@ Copy the current span of bytes.  C<copy> returns a new SecretBuffer object.  C<c
 into an existing buffer, which can be either a SecretBuffer or a scalar for non-secrets.  There
 is intentionally I<not> a method to I<return> a scalar, to avoid easily leaking secrets.
 
+=head2 memcmp
+
+  $cmp= $span->memcmp($other_thing);
+
+Compare contents of span byte-by-byte to another Span, SecretBuffer, or plain scalar.
+Returns the same as the 'cmp' operator.
+
 Options:
 
 =over
@@ -228,7 +235,7 @@ specify UTF-8 here, you will instead receive bytes of UTF-8 rather than perl wid
 
 =head1 VERSION
 
-version 0.011
+version 0.012
 
 =head1 AUTHOR
 
